/*
Copyright 2015 Google Inc. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package com.google.security.zynamics.binnavi.debug.debugger.synchronizers;

import com.google.security.zynamics.binnavi.CUtilityFunctions;
import com.google.security.zynamics.binnavi.Exceptions.MaybeNullException;
import com.google.security.zynamics.binnavi.debug.connection.packets.replies.ThreadClosedReply;
import com.google.security.zynamics.binnavi.debug.debugger.interfaces.IDebugEventListener;
import com.google.security.zynamics.binnavi.debug.debugger.interfaces.IDebugger;
import com.google.security.zynamics.binnavi.debug.models.processmanager.ProcessManager;
import com.google.security.zynamics.zylib.general.ListenerProvider;

/**
 * Synchronizes Thread Closed events from the debug client with the simulated target process.
 */
public final class ThreadClosedSynchronizer extends ReplySynchronizer<ThreadClosedReply> {
  /**
   * Creates a new Thread Closed synchronizer.
   *
   * @param debugger The debug client synchronize.
   * @param listeners Listeners that are notified about relevant events.
   */
  public ThreadClosedSynchronizer(final IDebugger debugger,
      final ListenerProvider<IDebugEventListener> listeners) {
    super(debugger, listeners);
  }

  @Override
  protected void handleSuccess(final ThreadClosedReply reply) {
    final ProcessManager processManager = getDebugger().getProcessManager();

    final long tid = reply.getThreadId();

    try {
      processManager.removeThread(processManager.getThread(tid));
    } catch (final MaybeNullException e) {
      CUtilityFunctions.logException(e);
    }
  }

  @Override
  protected void notifyListener(final IDebugEventListener listener, final ThreadClosedReply reply) {
    listener.receivedReply(reply);
  }
}
